# -*- coding: utf-8 -*-
import pygame

import data
import objects
import env
import render_image
import utils
import sound
import commands # Ugly hack for functions with special go methods
import introutro

area_image = pygame.Surface((800, 600)) # These are surfaces corresponding to the last updated state of the areas. We keep them here because they can't go with the area instances

# Area is HUGE. On the other hand, it handles almost anything you might wanna do
class Area:
    def __init__(self):
        self.name = "" # Unique identifier for area, this needs to be set twice, unfortunately. Once in areas.areas as well.

        self.background = "" # Filename for background

        self.bgmusic = ""
        
        self.paths = {} # Direction name: id of area in that direction. This also serves as a list of qualified functiion names

        self.visited = False # Has the player been here before?

        self.objects = {} # Dictionary of ID : instance, the reason for choosing an instance here is because the user may dropped modified objects

        # Dictionary of object_id : rect to place it on (a tuple also works). 
        # The last object will overlap the first one. Note that if you do not
        # wish to print an image for an object, don't reference it here.
        self.object_pos = {} 

        self.errors = {"not found" : "There is no %s here!",  # Standard error message for different functions
                       "take" : "You can't take that!",
                       "look_ahead" : "You can't go there!",
                       }
        
        self.long_desc = ""
        self.short_desc = ""

        self.local_functions = {} # Functions that are local to the area. Syntax: command name : name of function to call.
                                  # The reason we do not use command name : function to call is that we can't pickle function objects that way

    def print_objects(self):
        ret = ""
        for object in self.objects.itervalues():
            desc = object.location_description()
            if desc != "":
                ret += "\n" + desc
        if ret != "":
            ret = "\n" + ret
        return ret

    def description(self, short=False): # Note that this can be overridden if we want logic
    
        if short and not self.short_desc == "":
            ret =  self.short_desc # short description
        else:
            ret =  self.long_desc # long description
        ret += self.print_objects()

        return ret

    def _get_object(self, object):
        obj_id = objects.lookup(object)
        if obj_id is not None and obj_id in self.objects:
            return obj_id, self.objects[obj_id]
        return None, None

    def take(self, ob):
        id, obj = self._get_object(ob) # Get the object and its id, if any
        if obj is None:  # Could not find the correct object
            return self.errors["not found"] % ob
        elif not obj.takable: # This signifies a permanent object (which is really just semi-permanent; you can change takable to true)
            return self.errors["take"] # This can be more creative and dependin on the object, of course
        else:
            env.env["inventory"][id] = obj # This id is the "main id", and not an alias. Always use objects.lookup when comparing objects
            del self.objects[id] # Then we delete it from the area

            self.update() # Update the surface
            
            return "Taken."

    def put(self, name, object):
        """Object is object instance. Add instance to list of objects."""
        n = objects.lookup(name) # Just in case
        self.objects[n] = object

        self.update() # Since we might have modified some objects that we want to show

        return "Dropped."

    def look_ahead(self, direction):
        """Return the id of the area in direction"""
        if direction in self.paths: # If it's a qualified name for a direction
            fun = getattr(self, direction) # Get the corresponding function (same as the name)
            return fun() # Let it return the correct id
        else:
            return self.errors["look_ahead"] # This is not the same as the error messages from the direction functions.

    def enter(self, direction=None):
        """Do stuff that should be done/checked when entering an area. 
        Optional direction argument is the direction that the player 
        went in when coming to this area (i.e. not the direction from this 
        area's point of view)."""
        self.update() # Might lead to strange bugs otherwise, methinks

        if self.bgmusic is None:
                sound.stop_music()
        elif self.bgmusic: 
            sound.play_music(self.bgmusic)

        if not self.visited:
            self.visited = True
            return self.description(short=False)
        else:
            return self.description(short=True)
        
    def update(self): 
        global area_image
        objs = []
        for id in self.objects:
            if id in self.object_pos:
                objs.append((id, self.object_pos[id])) # Only display the objects that are present in the area
                # Note that dropped items are not shown, unless we expect a specific object to be
                # dropped, and place an entry in self.object_pos

        img = render_image.area_image(self.background, objs) # We pass the background filename and the list of object rects and id:s
        area_image = img # The suface is stored outside of the instance, only one is stored at a time
        
        return img

    def image(self):
        """Get cached image. Does not update if old."""
      
        return area_image

    # These are the standard direction functions. To add your own, add the funtion here and its name in self.paths 
    # (the value should be the neighbouring area. You can also set the value to None if you want to use the 
    # function, but only to create an event (e.g. rocks fall on your head if you try to climb up the attic.
    # if you cannot enter the attic, there is no need for an area.)
    # The reason all of these have their own function is that we might want to add logic to them
    def north(self):
        error = "You can't go that way!"
        if "north" in self.paths: # If the player is allowed to go north
            return self.paths["north"] # Return the id of the area to the north
        else:
            return error # Else, return an error message

    def south(self):
        error = "You can't go that way!"
        if "south" in self.paths: # If the player is allowed to go north
            return self.paths["south"] # Return the id of the area to the north
        else:
            return error # Else, return an error message

    def west(self):
        error = "You can't go that way!"
        if "west" in self.paths: # If the player is allowed to go north
            return self.paths["west"] # Return the id of the area to the north
        else:
            return error # Else, return an error message

    def east(self):
        error = "You can't go that way!"
        if "east" in self.paths: # If the player is allowed to go north
            return self.paths["east"] # Return the id of the area to the north
        else:
            return error # Else, return an error message
        
    def northwest(self):
        error = "You can't go that way!"
        if "northwest" in self.paths: # If the player is allowed to go north
            return self.paths["northwest"] # Return the id of the area to the north
        else:
            return error # Else, return an error message

    def northeast(self):
        error = "You can't go that way!"
        if "northeast" in self.paths: # If the player is allowed to go north
            return self.paths["northeast"] # Return the id of the area to the north
        else:
            return error # Else, return an error message

    def southwest(self):
        error = "You can't go that way!"
        if "southwest" in self.paths: # If the player is allowed to go north
            return self.paths["southwest"] # Return the id of the area to the north
        else:
            return error # Else, return an error message

    def southeast(self):
        error = "You can't go that way!"
        if "southeast" in self.paths: # If the player is allowed to go north
            return self.paths["southeast"] # Return the id of the area to the north
        else:
            return error # Else, return an error message
            
    def up(self):
        error = "You can't go that way!"
        if "up" in self.paths: # If the player is allowed to go north
            return self.paths["up"] # Return the id of the area to the north
        else:
            return error # Else, return an error message
            
    def down(self):
        error = "You can't go that way!"
        if "down" in self.paths: # If the player is allowed to go north
            return self.paths["down"] # Return the id of the area to the north
        else:
            return error # Else, return an error message
            
#
# End of Area
#

#
# Add rooms below
#

class roomX(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "roomx"
        self.background = "areax.jpg"

        self.paths = {"north" : "example_effects",
                      }

        self.objects = {"bajs" : objects.objs["bajs"](), # objects are placed in two dictionaries (three if you count the one with filenames). This returns the class
                        }
        
        self.object_pos = {"bajs" : (490, 450),
                           }

        self.local_functions = { "die" : "die", # Example, seems to be working
                                 } 
        
        self.long_desc = "You are standing in a room.\nThere is an impressionist painting on the wall.\nThere is knark on the floor"
        self.short_desc = "You are standing in a room."

    def enter(self, direction=None):
        sound.play_music("ghettotroll.ogg")
        return Area.enter(self, direction)


    def die(self, args):
        #return str(env.env["accomplishments"]["gork?"])
        return "hej"

    def description(self, short=False): # This is an example of how to override a function
        s = "Exampleamplepample\n"
        s += Area.description(self, short) # Note that you can still call the standard function, if you want
        
        return s

class withEffects(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "example_effects"
        self.background = "b2.jpg"

        self.paths = {"north" : "example_no_effects",
                      "south" : "roomx",
                      }

        self.objects = {
                        }
        
        self.object_pos = {
                           }
        

        self.long_desc = u"""Det här är en exempelbild som Krille gjorde.
Vad tycker ni om effekterna? Personligen gillar jag den enkla känslan som de ger, vilket jag tycker ger en större skala effekter (tänk mörk storstad), fast jag är osäker på hur det kommer att se ut i andra miljöer eller med människor.


Här är en väldigt lång och äcklig sträng, jag ska se om jag har gjort något fel på funktionen jag vet at tjag gjort ett fel

Söderut finns gamla hederliga roomx, norrut finns originalbilden för denna."""
        self.short_desc = u"Det här är en exempelbild som Krille gjorde."

    def enter(self, direction=None):
        sound.play_music("beginning.ogg")
        return Area.enter(self, direction)

class withoutEffects(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "example_no_effects"
        self.background = "b1.jpg"

        self.paths = {"south" : "example_effects",
                      }

        self.objects = {
                        }
        
        self.object_pos = {
                           }
        
        self.long_desc = u"""Originalbilden. 
Krille hälsar förövrigt att tråden inte gick att göra bättre på den här bilden.

Jag oroar mig äver för hur den fotade tråden ser ut med effekter..."""
        self.short_desc = "Originalbilden."

#
# Begin house
#

class yourCottage(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "your_cottage"
        self.background = "yourhouse.jpg"
        
        self.bgmusic = "home.ogg"

        self.paths = {"north" : "your_garden", 
                      "out" : "your_garden",
                      }

        self.objects = { "cardboard box" : objects.objs["cardboard box"](),
                         "food" : objects.objs["food"](),
                         "string" : objects.objs["string"](),
                         }

        self.local_functions = {"pull" : "pull",
                                "sleep" : "sleep",
                                "lie" : "sleep",
                                "sit" : "sleep",
                                
                                "go out" : "go_out",
                                "out" : "go_out",
                                "g out" : "go_out",
                                "g o" : "go_out",
                                "o" : "go_out",
                                }


    def go_out(self, args):
        utils.add_message(commands.go("out")) # Yes, this is ugly. It's pyweek...!

    def out(self):
        return self.paths["out"]

    def description(self, short=False):
        visited_garden = self.paths["north"] in env.env["areas"]
        
        if not visited_garden:
            s = "You just finished eating."
        else:
            s = "Your house."
            
        if not short:
            if "food" in self.objects:
                s += " The unfinished dinner is still on the table."
            if "cardboard box" in self.objects:
                s += " There is a cardboard box on the floor, next to the sofa that serves as your bed."
            else:
                s += " A very inviting looking sofa sits in a corner."
                
            s += " Next to your feet is the end of a red piece of string."
            s += "\n\nThe front door faces north"
            if visited_garden:
                s += ", to your garden."
            else:
                s += "."
        s += self.print_objects()

        return s

    def take(self, obj):
        obj_id = objects.lookup(obj)
        if obj_id == "string":
            return self.pull(["string"])
        else:
            return Area.take(self, obj)

    def pull(self, obj):
        obj_id = objects.lookup(obj[0])

        if obj_id == "string":
            s = "The string comes out a bit, but then it stubbornly resists."
            if not self.paths["north"] in env.env["areas"]:
                s += " Could it be stuck on the other side?"
            return s
        else:
            return "You can't do that."

    def sleep(self, obj):
        return "You lie down on the sofa for a while. Wow, you could do this all Pyweek!"

class yourGarden(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "your_garden"
        self.background = "yard.jpg"
        self.bgmusic = "home.ogg"
        
        self.paths = {"south" : "your_cottage",
                      "north" : "gate_from_house",
                      "southeast" : "tree_path_1",
                      }
        self.objects = {"string" : objects.objs["string"](),
                        }
        self.long_desc = """Your garden

In the far distance to the north, you can see the roofs of the city's tall stone buildings. The red string runs from the wall of your house on to a trail to the southeast."""

        self.short_desc = "Your garden"

#
# End house
#

#
# Begin southeastern branch
#

class treePath1(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "tree_path_1"
        self.background = "path2.jpg"
        self.bgmusic = "home.ogg"

        self.paths = {"northwest" : "your_garden",
                      "northeast" : "tree_path_2",
                      }
                      
        self.objects = {"string" : objects.objs["string"](),}
        
        self.long_desc = "The red string continues on a smaller path to the northeast. Your garden is to the northwest."

        self.short_desc = "Your cottage is to the northwest, the path continues northeast."

class treePath2(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "tree_path_2"
        self.background = "treefaraway.jpg"
        self.bgmusic = "home.ogg"

        self.paths = {"southwest" : "tree_path_1",
                      "northwest" : "camel_lands",
                      "northeast" : "tree_meadow",
                      }
        self.objects = {"string" : objects.objs["string"](),
                        }
        
        self.long_desc = "The path leads to the northwest, to an area with barbed wires and fences. The string runs up a tree to the northeast. If you go southwest, you will end up back at your cottage."

        self.short_desc = "The tree is to the northeast, the barbed wire to the northwest. The path to the southwest leads back to your cottage"

    def description(self, short=False):
        self.update()
        return Area.description(self,short)

class camelLands(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "camel_lands"
        self.background = "walruscamel.jpg"
        self.bgmusic = "camel.ogg"

        self.paths = {
                      }

        self.long_desc = "As you open the old, rusty gate, you hear a heavy panting sound as a viscious camel-walrus impales you with its teeth. Before the world blackens before your eyes, you see a gateopener lying before you.\nYou are dead. Type 'restart' to start over. I hope this is a lesson for you to save often."

        self.short_desc = "If you can see this, you are my hero."

    def enter(self, direction=None):
        if not "mustard gas" in env.env["inventory"]:
            env.env["current_area"] = areas["dead"] # Uh-oh! # One dies
            env.env["current_area"].enter()
            self.update()
            return self.long_desc
        else:
            env.env["inventory"]["gateopener"] = objects.objs["gateopener"]()
            del env.env["inventory"]["mustard gas"]
            env.env["current_area"] = areas["tree_path_2"] 
            self.update()
            return "You hear a panting sound and the ground begins to shake. The walrus plunges at you, and you fall backward, dropping your gas canister on the ground. The brown gas quickly fills the air and the camel-walrus lands flat on its stomache in a cry of displeassure.\n\nYou pick up the gateopener and leave before it's too late.\n\nYou return to the path."

class treeMeadow(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "Below the tree."
        self.background = "tree2.jpg"
        self.bgmusic = "home.ogg"

        self.paths = {"southwest" : "tree_path_2",
                      "up" : "tree",
                      }

        self.objects = {"string" : objects.objs["string"](),}
        
        self.long_desc = "You are standing in front of an old aspen tree. The string is intrically entangled in its higher branches.\n\nThe path you came from lies to the southwest."

        self.local_functions = { #"climb tree" : "go_up"
                                }


class tree(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "tree"
        self.background = "tree4.jpg"
        self.bgmusic = "home.ogg"

        self.paths = {"down" : "tree_meadow",
                      "up" : "tree_top",
                      }
        self.objects = {"string" : objects.objs["string"](),}
        
        self.long_desc = "Tree"
        
    def description(self,  short=False):
        s = "You have climbed halfway up the tree.\n"
        
        if not short:
            s += "You see that the string is jammed between a pair of branches. It continues to the higher branches."
            
        return s

class treeTop(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "tree_top"
        self.background = "treetop2.jpg"
        self.bgmusic = "home.ogg"

        self.paths = {"down" : "tree",
                      }

        self.long_desc = "Tree"
        
        self.objects = {"mustard gas" : objects.objs["mustard gas"](),
                        "string" : objects.objs["string"](),
                        }

        self.local_functions = {
                                "count camels" : "count_camels",
                                "count camelwalruses" : "count_camels",
                                "count camel-walruses" : "count_camels",
                                }


    def count_camels(selfself,  args):
        utils.add_message("There are 69105 bloodthirsty camel-walrus.")
        
    def description(self,  short=False):
        s = "You climb to the top of the tree."

        if not short:
            s += "\nThe string  continues through the air straight towards a city to the northwest. It passes the fences with barbed wires and finaly a farm of viscious, bloodthirsty camel-walrusess."

        s += self.print_objects()
            
        return s

#
# End of southeastern branch
#

#
# Begin road to town
#

class gateFromHouse(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "gate_from_house"
        self.background = "taube.jpg"
        self.bgmusic = "map.ogg"
        
        self.paths = {"south" : "your_garden",
                      "north" : "house_to_town_path_1",
                      }
        self.long_desc = """The gate. 

There is a gate to the north, beside which a man is standing."""

        self.local_functions = {"talk" : "talk",
                                "give" : "give", 
                                }
    
    def talk(self, args):
        if env.env["accomplishments"]["gate_opened"]:
            return "- Ah, those camel-walruses die too easily."
        else:
            return "- I would love to let you pass, but I dropped my gateopener in the camel cage. Could you get it for me?"

            
    def give(self, args):
        if len(args) > 0:
            name = args[0]
            base_name = objects.lookup(name) # Can have different aliases
            if base_name in env.env["inventory"]:
                item = env.env["inventory"][base_name]
                ret = self.put(base_name, item) # Add to current_area's items
                del env.env["inventory"][base_name] # Remove from inventory
                utils.add_message(ret)
            else:
                utils.add_message("You do not have that.")
        else:
            utils.add_message("Must specify an object.")
            
    def north(self):
        if env.env["accomplishments"]["gate_opened"]:
            return self.paths["north"]
        else:
            return "The gate is closed!"

    def put(self, name, obj):
        n = objects.lookup(name)
        if n == "gateopener":
            env.env["accomplishments"]["gate_opened"] = True
            return "The gatekeeper thanks you and opens the gate to the north."
        else:
            return Area.put(self, name, obj)

class houseToTownPath1(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "house_to_town_path_1"
        self.background = "pathtotown.jpg"
        self.bgmusic = "map.ogg"
        
        self.paths = {"south" : "gate_from_house",
                      "north" : "house_to_town_path_2",
                      }
        self.long_desc = """The path

You are standing on a road running north to a town in the distance. Above you the red string hangs in the air, heading north. Your house is to the south."""

class houseToTownPath2(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "house_to_town_path_2"
        self.background = "path9.jpg"
        self.bgmusic = "map.ogg"
        
        self.paths = {"south" : "house_to_town_path_1",
                      "northwest" : "western_valley_1",
                      "north" : "town_gates",
                      }
        self.long_desc = """The fork
The road forks of to the northwest. To the north, you can see the outskirts of the town, to where you can now clearly see the string is heading. Your house is south."""

#
# Begin northeastern branch
#

class westernValley1(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "western_valley_1"
        self.background = "path6.jpg"
        self.bgmusic = "litenand4.ogg"

        self.paths = {"southeast" : "house_to_town_path_2",
                      "northwest" : "western_valley_2",
                      }
                      
        self.objects = {"ladder" : objects.objs["ladder"](),
                }
        self.long_desc = """Forked road

The road leads far to the northwest, and according to the road signs, to an old junkyard. Going southeast will take you to the fork."""

        
class westernValley2(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "western_valley_2"
        self.background = "path4.jpg"
        self.bgmusic = "litenand4.ogg"
    
        self.paths = {"southeast" : "western_valley_1",
                      "northwest" : "western_valley_3",
                      }
        self.long_desc = """The road continues northwest, to the scrapyard. The forked road is to the southeast."""
        
class westernValley3(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "western_valley_3"
        self.background = "treeandrocks.jpg"
        self.bgmusic = "litenand4.ogg"

        self.paths = {"southeast" : "western_valley_2",
                      "north" : "western_valley_4",
                      }

        self.objects = {"tree branch" : objects.objs["tree branch"](),
                        }

        self.long_desc = "You have reached a hill. The road continues down to the north, to the scrapyard. The forked road lies to the southeast."

class westernValley4(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "western_valley_4"
        self.background = "nexttoruins.jpg"
        self.bgmusic = "litenand4.ogg"

        self.paths = {"south" : "western_valley_3",
                      "north" : "western_valley_5",
                      "west" : "western_valley_house",
                      }
        self.long_desc = "You are walking downhill. To the west you can see the remains of a small abandoned house, the path runs further downhill and north. The hill lies south."

class westernValleyHouse(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "western_valley_house"
        self.background = "ruins.jpg"
        self.bgmusic = "litenand4.ogg"

        self.paths = {"east" : "western_valley_4",
                      }

        self.objects = {"table leg" : objects.objs["table leg"](),
                        }

        self.objects = {"table leg" : objects.objs["table leg"](),
                        }
        
        self.long_desc = """Ruins

You stand among the ruins of a old house. Judging by the heavy vegetation, no one has lived here for a long time. The summit of the hill lies to the east."""

class westernValley5(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "western_valley_5"
        self.background = "pipes.jpg"
        self.bgmusic = "litenand4.ogg"

        self.paths = {"south" : "western_valley_4",
                      "north" : "western_valley_6",
                      "east" : "western_valley_7",
                      }
        self.long_desc = "The hill is to the south. Before you runs a road connecting the two scrapheaps to the east and to the north."

class westernValley6(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "western_valley_6"
        self.background = "scrapheap.jpg"
        self.bgmusic = "litenand4.ogg"

        self.paths = {"south" : "western_valley_5",
                      }

        self.objects = {"pipe" : objects.objs["pipe"](),
                        }

        self.long_desc = """Northern scrapheap

This is a dead end, you can go south to go back."""

class westernValley7(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "western_valley_7"
        self.background = "buildingsite.jpg"
        self.bgmusic = "litenand4.ogg"

        self.paths = {"west" : "western_valley_5",
                      }
        self.objects = {"flashlight" : objects.objs["flashlight"](),
                        }

        self.long_desc = """The eastern part of the scrapyard

In the background, you can see the city's tall buildings. You cannot find a road going there, though."""

#
# End northeastern branch
#

#
# Begin town
#

class townGates(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "town_gates"
        self.background = "citygates2.jpg"
        self.bgmusic = "map.ogg"

        self.paths = {"east" : "high_wall",
                      "south" : "house_to_town_path_2",                      
                      }

        self.local_functions = {"knock" : "knock",
                                "l door": "knock",
                                "look door" : "knock",
                                "open" : "knock",
                                "use door" : "knock",
                                }

        self.long_desc = "From behind the wall you can clearly hear the sounds of the rushed town life. The gate is closed and locked.\n\nA path runs along the wall to the east. The road to the south will take you to your house."

    def knock(self, args):
        return "You knock the door, and a stern voice responds that it is down for maintenance."

#
# Begin bakery 
#

class highWall(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "high_wall"
        self.background = "wall.jpg"
        self.bgmusic = "map.ogg"

        self.paths = {"west": "town_gates", 
                      "east" : "high_wall2",
                      }

        self.long_desc = "You are standing outside the city walls. The path goes to the east and the west."

class highWall2(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "high_wall2"
        self.background = "wall.jpg"
        self.bgmusic = "map.ogg"

        self.paths = {"west": "high_wall", 
                      "east" : "low_wall",
                      }

        self.long_desc = "You are standing outside the city walls. The path goes to the east and the west."


class lowWall(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "low_wall"
        self.background = "wall.jpg"
        self.bgmusic = "map.ogg"

        self.paths = {"west": "high_wall2", 
                      "east" : "high_wall3",
                      "north" : "inside_city_house",
                      "up" : "inside_city_house",
                      }

        self.long_desc = "This section of the wall is slightly lower. The path goes to the east and the west."

    def north(self):
        if env.env["accomplishments"]["used_ladder"]:
            return self.paths["north"]
        else:
            return "You can't go there!"

    def up(self):
        if env.env["accomplishments"]["used_ladder"]:
            return self.paths["up"]
        else:
            return "You can't go there!"

    def description(self, short=False):
        s = Area.description(self, short)
        if env.env["accomplishments"]["used_ladder"]:
            s += "\n\nA ladder is leaning towards the wall."
        return s 

class highWall3(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "high_wall3"
        self.background = "wall.jpg"
        self.bgmusic = "map.ogg"

        self.paths = {"west" : "low_wall",
                      "south" : "bakery_outside",
                      }

        self.long_desc = "You are standing outside the city walls. The path goes to the west. A faint odor is coming from the south."



class bakeryOutside(Area):
    def __init__(self):
        Area.__init__(self)
        self.name = "bakery_outside"
        self.background = "bakeryoutside.jpg"
        self.bgmusic = "home.ogg"

        self.paths = {"north" : "high_wall3",
                      "inside" : "bakery_inside",
                      }

        self.local_functions = { "go in" : "go_in",
                                 "g in" : "go_in",
                                 "in" : "go_in",
                                 "enter" : "go_in",
                                 }

        self.long_desc = "You are standing outside a small bakery. The smell of fresh bread makes you hungry. The city wall lies to the north."

    def go_in(self, args):
        utils.add_message(commands.go("inside")) # Yes, this is ugly. It's pyweek...!

    def inside(self):
        return self.paths["inside"]


class bakeryInside(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "bakery_inside"
        self.background = "bakeryinside.jpg"
        self.bgmusic = "home.ogg"

        self.paths = {"out" : "bakery_outside",
                      }

        self.local_functions = { "go out" : "go_out",
                                 "g out" : "go_out",
                                 "out" : "go_out",
                                 "exit" : "go_out",

                                 "talk" : "talk_girl",
                                 }

        self.long_desc = "There are piles of bread stacked on the counters. There is a cute girl inside."

    def go_out(self, args):
        utils.add_message(commands.go("out")) # Yes, this is ugly. It's pyweek...!

    def out(self):
        return self.paths["out"]

    def talk_girl(self, args):
        env.env["inventory"]["baguette"] = objects.objs["baguette"]()
        return "-Welcome, stranger. You look hungry. Here, have a baguette.\n\nReceived a baguette!"

#
# End bakery     
# 

#
# Begin inside of city
#
class insideCityHouse(Area):
   def __init__(self):
        Area.__init__(self)

        self.name = "inside_city_house"
        self.background = "secondfloor.jpg"
        self.bgmusic = "town.ogg"

        self.paths = {"south" : "low_wall",
                      "down" : "city_street_1",
                      }

        self.long_desc = "You are standing on the second floor of a small house. The window from which you entered is to the south. You hear the house's owners from downstairs."



class cityStreet1(Area):
   def __init__(self):
        Area.__init__(self)

        self.name = "city_street_1"
        self.background = "city1.jpg"
        self.bgmusic = "town.ogg"

        self.paths = {"east" : "inside_city_house",
                      "west" : "city_street_2",
                      "north" : "city_square",
                      }

        self.long_desc = "You are standing on the street in front of a small house. All around you, you can see tall buildings. The street continues to the west. The city gates are to the south."

class cityStreet2(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "city_street_2"
        self.background = "city2.jpg"
        self.bgmusic = "town.ogg"

        self.paths = {"east" : "city_street_1",
                      "west" : "city_street_3",
                      }
                    
        self.long_desc = """Southern back alley

The street goes to the west and the east."""
    
class cityStreet3(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "city_street_3"
        self.background = "city3.jpg"
        self.bgmusic = "town.ogg"


        self.paths = {"east" : "city_street_2",
                      "north" : "city_street_4",
                      }
                    
        self.long_desc = """Curve

The street goes to the north and to the east."""

class cityStreet4(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "city_street_4"
        self.background = "city4.jpg"
        self.bgmusic = "town.ogg"
        
        self.paths = {"south" : "city_street_3",
                      "east" : "city_square",
                      }
                    
        self.long_desc = """The street goes south to a back alley and to the east, toward the central square."""

class cityStreet5(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "city_street_5"
        self.background = "city5.jpg"
        self.bgmusic = "town.ogg"


        self.paths = {"west" : "city_square",
                      "north" : "city_street_6",
                      }
                    
        self.long_desc = """The alley continues to the north. Going west will take you to the central square."""

class cityStreet6(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "city_street_6"
        self.background = "city6.jpg"
        self.bgmusic = "town.ogg"


        self.paths = {"west" : "city_street_7",
                      "south" : "city_street_5",
                      }
                    
        self.long_desc = """Northeastern backalley

The alleyway continues to the west and the south."""

class cityStreet7(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "city_street_7"
        self.background = "manhole.jpg"
        self.bgmusic = "town.ogg"

        self.paths = {"east" : "city_street_6",
                      "south" : "city_square",
                      "down" : "sewer",
                      }
                    
        self.long_desc = """There is a manhole in the ground. The road goes to the east and to the south."""

class citySquare(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "city_square"
        self.background = "citysquare.jpg"
        self.bgmusic = "town.ogg"

        self.paths = {"south" : "city_street_1",
                      "north" : "city_street_7",
                      "west" : "city_street_4",
                      "east" : "city_street_5",
                      }
                    
        self.long_desc = """Central square

A square that connects all the town's streets."""

            
class sewer(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "sewer"
        self.background_true = "tunnel1.jpg"
        self.background = "tunnel1.jpg"

        self.bgmusic = "ghettotroll.ogg"

        self.paths = {"up" : "city_street_7",
                      "north" : "sewer2",
                      }

        self.local_functions = { "use flashlight":"use_flashlight"}
        
        self.long_desc = "It is dark. A horrible stench surrounds you."
    
    def use_flashlight(self,  args):
        if "flashlight" in env.env["inventory"] :
            env.env["inventory"]["flashlight"].use()
            self.enter()
            
    def enter(self, direction=None):
        sound.play_music(self.bgmusic)
        if "flashlight" in env.env["inventory"] and env.env["inventory"]["flashlight"].on:
            self.background = self.background_true
            self.paths["north"] = "sewer2"
            self.update()
            return Area.enter(self, direction)
        else:
            self.background = "darkroom.jpg"
            self.update()
            self.paths["north"] = "grue"
            return "It's dark. You are likely to be eaten by a grue. The path continues to the north."

class sewer2(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "sewer2"
        self.background = "endofsewers.jpg"

        self.bgmusic = "ghettotroll.ogg"

        self.paths = {"up" : "path_to_forest_1",
                      "north" : "path_to_forest_1",
                      "south" : "sewer",
                      }

        self.long_desc = "The string comes out of an airvent behind you and goes up the stairs to the north."
            
#
# End town
#

#
#Begin path to forest
#
class pathToForest1(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "path_to_forest_1"
        self.background = "path7.jpg"
        self.bgmusic = "ghettotroll.ogg"

        self.objects = {"string": objects.objs["string"](), }
        
        self.paths = {"south" : "sewer2",
                      "north" : "path_to_forest_2",
                      }

        self.long_desc = "You are standing on a path. The string continues north."
    
        self.short_desc = "The path from the sewers"
        
class pathToForest2(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "path_to_forest_2"
        self.background = "path5.jpg"
        self.bgmusic = "ghettotroll.ogg"

        self.objects = {"string": objects.objs["string"](), }
        
        self.paths = {"west" : "forest_entrance",
                      "east" : "camel_farm",
                      "south" : "path_to_forest_1"
                      }

        self.long_desc = "You are standing at a crossroads. The red string make a turn for the west. There is also a path to the east, and to the south you see the town."
        
        self.short_desc = "The crossroads"

class forestEntrance(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "forest_entrance"
        self.background = "forestentry.jpg"
        self.bgmusic = "ghettotroll.ogg"

        self.objects = {"string": objects.objs["string"](), 
                        }
        
        self.local_functions = { "go in" : "go_in",
                                 "g in" : "go_in",
                                 "in" : "go_in",
                                 "enter" : "go_in",
                                 }
        
        self.paths = {"east" : "path_to_forest_2",
                      "north" : "forest1",
                      "inside" : "forest1",
                      }

        self.short_desc = "The entrance to the forest"
        self.long_desc = "You are standing at the edge of a forest. The red string disappears under a large root. The crossroads is the the east."
    
    def go_in(self, args):
        utils.add_message(commands.go("inside")) # Yes, this is ugly. It's pyweek...!

    def inside(self):
        return self.paths["inside"]
    
    

        
class camelFarm(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "camel_farm"
        self.background = "walruscamel.jpg"
        self.bgmusic = "camel.ogg"

        self.objects = {"compass": objects.objs["compass"](), }
        
        self.paths = {"west" : "path_to_forest_2",
                      }

        self.long_desc = "You are standing outside the gates of the camel-walrus farm. The road to the west leads out of here."
        
        self.short_desc = "The camel-walrus farm"

        
#
#End path to forest
#


#
# Begin forest
#

class forest1(Area):
    def __init__(self):
        Area.__init__(self)
        self.name = "forest1"

        self.background = "forest1.jpg"

        self.paths_false = {"north" : "forest1",
                           "west" : "forest1",
                           "east" : "forest1",
                           "south" : "forest_entrance",
                           
                           "northwest" : "forest1",
                           "southwest" : "forest1",
                           "northeast" : "forest1",
                           "southeast" : "forest1",
                           }

        self.paths_true = {"north" : "forest2",
                           "west" : "forest2",
                           "east" : "forest2",
                           "south" : "forest2",
                           
                           "northwest" : "forest2",
                           "southwest" : "forest2",
                           "northeast" : "forest5",
                           "southeast" : "forest5",
                           }

        self.long_desc = "Its dark. All trees look alike. You hear strange voices and laughter all around you."

    def enter(self):
        if not "compass" in env.env["inventory"]:
            self.paths = self.paths_false
        else:
            self.paths = self.paths_true
        
        sound.play_music("forest.ogg")
        self.update()
        return self.long_desc # We circumvent short descriptions

class forest2(Area):
    def __init__(self):
        Area.__init__(self)
        self.name = "forest2"

        self.background = "forest2.jpg"

        self.paths = {"north" : "forest3",
                      "west" : "forest1",
                      "east" : "forest3",
                      "south" : "forest1",

                      "northwest" : "forest1",
                      "southwest" : "forest1",
                      "northeast" : "forest1",
                      "southeast" : "forest1",
                      }

        self.long_desc = "Its dark. All trees look alike. You hear strange voices and laughter all around you."

    def enter(self):
        sound.play_music("forest.ogg")
        self.update()
        return self.long_desc # We circumvent short descriptions

class forest3(Area):
    def __init__(self):
        Area.__init__(self)
        self.name = "forest3"

        self.background = "forest3.jpg"

        self.paths = {"north" : "forest1",
                      "west" : "forest2",
                      "east" : "forest2",
                      "south" : "forest1",

                      "northwest" : "forest2",
                      "southwest" : "forest1",
                      "northeast" : "forest1",
                      "southeast" : "forest1",
                      }

        self.long_desc = "Its dark. All trees look alike. You hear strange voices and laughter all around you."

    def enter(self):
        sound.play_music("forest.ogg")
        self.update()
        return self.long_desc # We circumvent short descriptions

class forest4(Area):  # This is the important place
    def __init__(self):
        Area.__init__(self)
        self.name = "forest4"
        self.background = "forest4.jpg"

        self.paths = {"north" : "forest2",
                      "west" : "forest2",
                      "east" : "forest2",
                      "south" : "forest2",

                      "northwest" : "forest3",
                      "southwest" : "forest2",
                      "northeast" : "forest3",
                      "southeast" : "forest2",
                      }

        self.long_desc = "Its dark. All trees look alike. You hear strange voices and laughter all around you."

    def enter(self):
        sound.play_music("forest.ogg")
        self.update()
        return self.long_desc # We circumvent short descriptions

class forest5(Area): # The exit back is here
    def __init__(self):
        Area.__init__(self)
        self.name = "forest5"

        self.background = "forest5.jpg"

        self.paths = {"north" : "forest3",
                      "west" : "forest2",
                      "east" : "forest2",
                      "south" : "forest_entrance",

                      "northwest" : "forest2",
                      "southwest" : "forest6",
                      "northeast" : "forest3",
                      "southeast" : "forest3",
                      }

        self.long_desc = "Its dark. All trees look alike. You hear strange voices and laughter all around you."

    def enter(self):
        sound.play_music("forest.ogg")
        self.update()
        return self.long_desc # We circumvent short descriptions

class forest6(Area): # The exit to the mountains is here
    def __init__(self):
        Area.__init__(self)
        self.name = "forest6"

        self.background = "forest6.jpg"

        self.paths = {"north" : "forest2",
                      "west" : "glade",
                      "east" : "forest2",
                      "south" : "forest2",

                      "northwest" : "forest2",
                      "southwest" : "forest2",
                      "northeast" : "forest2",
                      "southeast" : "forest2",
                      }

        self.long_desc = "Its dark. All trees look alike. You hear strange voices and laughter all around you."

    def enter(self):
        sound.play_music("forest.ogg")
        self.update()
        return self.long_desc # We circumvent short descriptions
    
class glade(Area):
    def __init__(self):
        Area.__init__(self)
        self.name = "glade"
        self.background = "lyingonaboulder.jpg"
        self.bgmusic = "forest.ogg"

        self.objects = {"tablets" : objects.objs["tablets"](), 
                        "pair of scissors" : objects.objs["pair of scissors"](), 
                        }
        self.local_functions = { "read":"read"}
                         
        self.paths = {"east":"forest6", 
                      "west":"ravine"
                      }

        self.long_desc = "You are standing in a small glade in the forest. To the west you see an opening between the trees. There are five stone tablets here. "
    
    def read(self,  args):
        return self.objects["tablets"].description()

class ravine(Area):
    def __init__(self):
        Area.__init__(self)
        self.name = "ravine"
        self.background = "ullisending.jpg"
        self.bgmusic = None

        self.paths = {"east":"glade",
                      "north" : "ravine2",
                      }

        self.long_desc = "You are standing next to a big ravine, where the cute girl is standing. The red thread emerges from under a root and continues north towards the ravine. The forest is to the east."
        
        self.local_functions = { "talk" : "talk_girl",
                                 }

    def talk_girl(self, args):
        return """You walk up to the ravine. The girl turns around when she hears you, and then turns back to gaze to the north.

- This place is so beautiful. I wish I could stay here for the rest of my life, the girl says. She picks up the string and looks at it for a while. Then, she turns over to you and drops the string.

- Did you find out what really matters to you in the forest? She makes a short pause as if to wait for an answer, but then she looks down. Many people are afraid of that forest, but I know it speaks to your soul. It tells you what really matters in life, what really is important.

The girl blushes and draws circles with her foot. She says nothing for a while and all you hear is the wind blowing.

- We make our own meaning in life my dad used to say. Now I know what that means. She looks up again and into your eyes. The wind sweeps her hair and puts tears in her eyes."""

class ravine2(Area):
    def __init__(self):
        Area.__init__(self)
        self.name = "ravine2"
        self.background = "ravine.jpg"
        self.bgmusic = None

        self.paths = {"south" : "ravine",
                      }

        self.long_desc = "The string runs down into the ravine. It is tense, as if something is weighing it down."
        
        self.local_functions = { "cut string" : "cut_string", 
                                 "use scissors" : "cut_string", 
                                 
                                 "climb" : "climb", 
                                 "climb down" : "climb",
                                 "climb d" : "climb",
                                 }

    def cut_string(self, args):
        if "pair of scissors" in env.env["inventory"]:
            env.env["current_area"] = areas["credits"]
            return env.env["current_area"].enter("ravine2.jpg")
        else:
            return "You don't have anything to cut the string with!"
        
    def climb(self, args):
        env.env["current_area"] = areas["credits"]
        return env.env["current_area"].enter("climbingdown.jpg")
        

class credits(Area): # The exit to the mountains is here
    def __init__(self):
        Area.__init__(self)
        self.name = "credits"

        self.background = "theend.jpg"

        self.paths = {
                      }

        self.long_desc = "Congratulations! \n\nType 'restart' to play again, or 'quit' to turn it off."

    def enter(self, image):
        introutro.play_credits(image)
        self.update()
        env.env["current_area"] = areas["dead"] # Uh-oh! # One dies
        env.env["current_area"].enter()
        #sound.stop_music()
        self.update()
        return self.long_desc

#
# End forest
#

class dead(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "dead"
        self.background = "theend.jpg"

        self.paths = { # Player can't get out
                      }
        self.long_desc = "You are dead. All is black. Restart the game by typing 'restart'"

        self.local_functions = {"restart" : "restart",
                                }
                                    
                                    
    def enter(self,  direction=None):
        ret = Area.enter(self,  direction)
        #sound.play_music("dead.ogg")
        #pygame.time.wait(2500)
        sound.stop_music()
        
        return ret
        
    def restart(self, args):
        for key, val in env.env.iteritems(): # FUCKING FUCK I HATE YOU, YOU FUCKING DICTIONARIES!
            if isinstance(val, dict):
                ks = []
                for k,v in val.iteritems():
                    ks.append(k)
                for k in ks:
                    del val[k]
            else:
                env.env[key] = env.orig_env[key]

        for key, val in areas_orig.iteritems():  # To fix the death bug
            areas[key] = val()
        #env.env["areas"] = {}
        #env.env["accomplishments"] = collections.defaultdict(env.default)
        #env.env["current_area"] = areas.areas["your_house"]

        #for key,value in env.orig_env.iteritems(): # Restore all fields of env
        #    if isinstance(value, dict):
        #        env.env[key] = value.copy()
        #    else:
        #        env.env[key] = value
        #env.env = env.orig_env.copy()
        #areas = areas_orig.copy()
        env.env["current_area"] = areas["your_cottage"] # Classes are dictionaries, have to be renewed

        # I don't have time to clean this shit up

        return env.env["current_area"].enter()

class grue(Area):
    def __init__(self):
        Area.__init__(self)

        self.name = "grue"
        self.background = "darkroom.jpg"

        self.paths = {
                      }

        self.long_desc = "As you walk through the darkness, a grue sneaks up on you and swallows huge chunks of your flesh. \n\nYou are dead. Type 'restart' to play again."

        self.short_desc = "If you can see this, you are a sexy beast."

    def enter(self, direction=None):
        env.env["current_area"] = areas["dead"] # Uh-oh! # One dies
        env.env["current_area"].enter()
        self.update()
        return self.long_desc
            

# Identifiers for areas
areas_orig = {"roomx" : roomX,
              "example_effects" : withEffects,
              "example_no_effects" : withoutEffects,
              
              "your_cottage" : yourCottage,
              "your_garden" : yourGarden,
              
              "tree_path_1" : treePath1,
              "tree_path_2" : treePath2,
              "camel_lands" : camelLands,
              
              "tree_meadow" : treeMeadow,
              "tree" : tree,
              "tree_top" : treeTop,
              
              "gate_from_house" : gateFromHouse,
              "house_to_town_path_1" : houseToTownPath1,
              "house_to_town_path_2" : houseToTownPath2,
              
              "western_valley_1" : westernValley1,
              "western_valley_2" : westernValley2,
              "western_valley_3" : westernValley3,
              "western_valley_4" : westernValley4,
              "western_valley_house" : westernValleyHouse,
              "western_valley_5" : westernValley5,
              "western_valley_6" : westernValley6,
              "western_valley_7" : westernValley7,
              
              "town_gates" : townGates,
              "high_wall" : highWall,
              "high_wall2" : highWall2,
              "low_wall" : lowWall,
              "high_wall3" : highWall3,
              "bakery_outside" : bakeryOutside,
              "bakery_inside" : bakeryInside,
              "inside_city_house" : insideCityHouse, 
              "city_street_1" : cityStreet1, 
              "city_street_2" : cityStreet2, 
              "sewer" : sewer, 
              
              "inside_city_house" : insideCityHouse,
              "city_street_1" : cityStreet1,
              "city_street_2" : cityStreet2,
              "city_street_3" : cityStreet3,
              "city_street_4" : cityStreet4,
              "city_street_5" : cityStreet5,
              "city_street_6" : cityStreet6,
              "city_street_7" : cityStreet7,
              "city_square" : citySquare,
              "sewer" : sewer,
              "sewer2" : sewer2,

              "path_to_forest_1" : pathToForest1, 
              "path_to_forest_2" : pathToForest2, 
              "forest_entrance" : forestEntrance, 
              "camel_farm" :camelFarm, 
              
              "forest1" : forest1,
              "forest2" : forest2,
              "forest3" : forest3,
              "forest4" : forest4,
              "forest5" : forest5,
              "forest6" : forest6,
              
              "glade" : glade, 
              "ravine" : ravine, 
              "ravine2" : ravine2,

              "credits" : credits,
              
              "grue" : grue,
              "dead" : dead,
              }

areas = {}

for key, val in areas_orig.iteritems():  # To fix the death bug
    areas[key] = val()
