# commands.py
# -*- coding: utf-8 -*-
# All command functions

import pygame

import env
import utils
import sound
import areas
import objects

# All commands accept a list of arguments
def go(direction):
    """Note that the direction is parsed along with the go user command. 
    This is to make the "go" part optional."""
    
    area = env.env["current_area"]
    next_area_id = area.look_ahead(direction)

    env.save_current_area_state() # Do this regardless of whether we can move. It needs to be done anyways, and it doesn't hurt.
        
    # Protip: Don't return error messages with the same contents as an area id
    if next_area_id in env.env["areas"]: # If we already have such an area with altered state
        env.env["current_area"] = env.env["areas"][next_area_id]
        desc = env.env["current_area"].enter()
        utils.add_message(desc)
    elif next_area_id in areas.areas:    # Grab a new instance of that area type
        env.env["current_area"] = areas.areas[next_area_id]
        desc = env.env["current_area"].enter()
        utils.add_message(desc)
    else: # In this case, it's probably an error message.
        utils.add_message(next_area_id)
        desc = ""

    return desc # For compatibility with local_functions that call go

def take(args):
    if len(args) > 0:
        obj = " ".join(args)
        utils.add_message(env.env["current_area"].take(obj))
    else:
        utils.add_message("Take what?")

def drop(args):
    if len(args) > 0:
        name = " ".join(args)
        base_name = objects.lookup(name) # Can have different aliases
        if base_name in env.env["inventory"]:
            item = env.env["inventory"][base_name]
            ret = env.env["current_area"].put(base_name, item) # Add to current_area's items
            del env.env["inventory"][base_name] # Remove from inventory
            utils.add_message(ret)
        else:
            utils.add_message("You do not have that.")
    else:
        utils.add_message("Must specify an object.")

def inventory(args):
    items = ""
    for name in env.env["inventory"]:
        obj = env.env["inventory"][name]
        items += "You are carrying " + obj.fullname() + ".\n"
    utils.add_message(items[:-1]) # Remove the last linebreak
                                     
def put(args):
    """Put objectA [in | on | into] objectB."""
    pass

def use(args):
    """Use objectA
    Use objectA [on | with] objectB"""

    a_alias = ""
    b_alias = ""

    for num,arg in enumerate(args): # The first object is every word up till "with" or "on"
        if arg == "with" or arg == "on":
            a_alias = " ".join(args[:num])
            b_alias = " ".join(args[num + 1:])
    if not a_alias:
        a_alias = " ".join(args) # Only one item was supplied
            
    a_name = objects.lookup(a_alias)
    obj_a = None

    if a_name in env.env["current_area"].objects:
        obj_a = env.env["current_area"].objects[a_name]
    elif a_name in env.env["inventory"]:
        obj_a = env.env["inventory"][a_name]
    else:
        utils.add_message("You do not have %s." % a_alias)
        return

    if b_alias:
        b_name = objects.lookup(b_alias)

        if b_name in env.env["current_area"].objects:
            obj_b = env.env["current_area"].objects[b_name]
        elif b_name in env.env["inventory"]:
            obj_b = env.env["inventory"][b_name]
        else:
            utils.add_message("You do not have %s." % b_alias)
            return

        ret = obj_b.use(a_name) # Use objectA on objectB

    else:
        ret = obj_a.use() # Just use it on its own

    utils.add_message(ret)
    

def help(args):
    s = """Welcome, adventurer!

Basic controls for the interface:
  ESC -- Quit the game
  Enter -- Input command
  Page Up -- Scroll back in message history
  Page Down -- Scroll forward in message history
  Up -- Scroll back in input history
  Down -- Scroll forward in input history
  Left/right -- move cursor

  Ctrl-M -- mute sound (equivalent to the "mute" command)
  Alt-Enter -- toggle fullscreen
  Ctrl-F -- toggle fullscreen (same as Alt-Enter)


You may save your progress with the "save" command. Type "save filename" to 
save to a file named filename. You may then restore your progress by typing 
"load filename\""""

    utils.add_message(s)

def save(args):
    if len(args)>0:
        # I am too lazy to account for surrounding " characters, so let's just concatenate the args
        ret = utils.save_game(" ".join(args)) 
        utils.add_message(ret)
    else:
        utils.add_message("You must specify a filename.")
def load(args):
    if len(args)>0:
        # I am too lazy to account for surrounding " characters, so let's just concatenate the args
        errcode,ret = utils.load_game(" ".join(args)) 
        utils.add_message(ret)
    else:
        utils.add_message("You must specify a filename.")
def examine(args):
    if len(args) is 0:
        utils.add_message(env.env["current_area"].description())
    else: 
        base_name = objects.lookup(" ".join(args))
        if base_name in env.env["inventory"]:
            utils.add_message(env.env["inventory"][base_name].description())
        elif base_name in env.env["current_area"].objects:
            utils.add_message(env.env["current_area"].objects[base_name].description()) # This uses the obejct's own method and cannot be scripted on a per area basis
        else:
            if base_name is not None: #quick ugly fix
                utils.add_message("You don't see any %s in here." % base_name)
            else: 
                utils.add_message("You can't see that.")


# Functions can be defined on a per area basis. Look up such a function here by name
def local_function(name, args):
    # Because we can't pickle functions directly (as in "name" : fun), we 
    # have a dictionary of command_name : function_name
    if name in env.env["current_area"].local_functions: # If we are allowed to call it
        funname = env.env["current_area"].local_functions[name] # Get the name of the function
        fun = getattr(env.env["current_area"], funname) # look it up
        ret = fun(args)
        if ret is not None:
            utils.add_message(ret) # and call it
        return True # We did in fact find such a function
    else:
        return False # Function was not found

def mute(args):
    sound.toggle_mute()
def fullscreen(args):
    pygame.display.toggle_fullscreen()

def wazzamo(state):
    # put wazzamo in state
    env.env["wazzamo"] = state

def warp(args):
    if env.env["wazzamo"]:
        if len(args) is 0:
            utils.add_message("Please specify a location.")
            return
        
        area_id = args[0]

        env.save_current_area_state() # Do this regardless of whether we can move. It needs to be done anyways, and it doesn't hurt.
        
        if area_id in env.env["areas"]:   # Works the same way as go() above
            env.env["current_area"] = env.env["areas"][area_id]
            desc = env.env["current_area"].enter()
            utils.add_message(desc)
        elif area_id in areas.areas:
            env.env["current_area"] = areas.areas[area_id]
            desc = env.env["current_area"].enter()
            utils.add_message(desc)
        else: # invalid place name
            utils.add_message("You chant the spell of teleportation. You fail. You are a naughty wazza.")

    else:
        utils.add_message("You can't do that. Seriously.")

def spawn_item(args):
    if env.env["wazzamo"]:
        # Since there is no support for multiple items of the same type on a 
        # single area or in the inventory, this is just a way to get boring items quick
        alias = " ".join(args)
        name = objects.lookup(alias)
        if name in objects.objs:
            obj = objects.objs[name]() # Make a new copy
            env.env["current_area"].put(name, obj)
            utils.add_message(env.env["current_area"].description())
        else:
            utils.add_message("No such item.")
    else:
        utils.add_message("You can't do that. Seriously.")

def getenv(args):
    if env.env["wazzamo"] and len(args)>0:
        name = args[0] # For compatibility with setenv, variables can only be one word
        value = env.env["accomplishments"][name] # 'tis a defaultdict, so this is ok
        utils.add_message("[%s] is %s." % (name, str(value)))
    else:
        utils.add_message("You can't do that. Seriously.")

def setenv(args):
    if env.env["wazzamo"] and len(args)>0:
        name = args[0]
        
        if len(args) is not 2:
            utils.add_message("Wrong number of arguments!")
            return

        value = args[1]
        if value == "false" or value == "f" or value == "0" or value == "nil":
            value = False
        elif value == "true" or value == "t" or value == "1":
            value = True
        else:
            utils.add_message("Invalid value!")
            return

        env.env["accomplishments"][name] = value 
        utils.add_message("[%s] is %s." % (name, str(value)))
    else:
        utils.add_message("You can't do that. Seriously.")
