# gui.py
# -*- coding: utf-8 -*-
# Widgets for controlling the game

import pygame
from pygame.locals import *
import math

import utils
#import parser

gui_box_color = (0,0,0)#200, 200, 200)
#box_border_color = (0, 0, 0)
#box_border_w = 3
box_alpha = 200
box_text_color = (253, 149, 15)#173, 48, 15)

gui_font = "uglyqua"
gui_italic = False
gui_bold = False
gui_font_size = 16

class MessageBox(pygame.sprite.Sprite):
    def __init__(self, position, size, text_size=10):
        pygame.sprite.Sprite.__init__(self)

        self.rect = pygame.Rect(position, size) #this is just the initial size
        self.messages = []
        self.message_index = 0
        
        self.box_padding = 20

        # Just print an empty message that will not show up in the history
        self.image = wrap_in_box(render_text("", self.rect.w - 2*self.box_padding), self.rect.size)
                     
    def add(self, string):
        self.messages.append(string)
        self.message_index = len(self.messages) - 1

        self.update()

    def update(self):
        message = self.messages[self.message_index]
        text_surf = render_text(message + "\n", self.rect.w - 2*self.box_padding, bg_color=gui_box_color) # The newline is to get the indicator to not overlap

        # Resize box to match text
        text_rect = text_surf.get_rect()

        oldcenterx = self.rect.centerx
        oldbottom = self.rect.bottom

        h = text_rect.h
        w = self.rect.w

        self.rect = pygame.Rect((0, 0), (w, h + 2*self.box_padding))

        # And position the box correctly
        self.rect.centerx = oldcenterx
        self.rect.bottom = oldbottom
        
        # Then we put a box around it. 
        self.image = wrap_in_box(text_surf,
                                 self.rect.size,
                                 padding=self.box_padding)
        # An indicator for the message index
        ind_surf = render_text(str(self.message_index + 1) + "/" + str(len(self.messages)), self.rect.w - self.box_padding, bg_color=gui_box_color)
        
        ind_rect = ind_surf.get_rect()
        ind_rect.bottomright = (self.rect.w - self.box_padding, self.rect.h - self.box_padding)
        self.image.blit(ind_surf, ind_rect)
                                
    def move_history(self, step):
        """Move by step in message history"""
        newindex = self.message_index + step
        if 0 <= newindex and newindex < len(self.messages):
            self.message_index = newindex
        self.update()

class InputBox(pygame.sprite.Sprite):
    def __init__(self, parser, position, size):
        pygame.sprite.Sprite.__init__(self)
        
        self.parser = parser
        
        self.input = []
        self.history = []
        self.history_index = 0

        self.curs_pos = 0
        self.rect = pygame.Rect(position, size)
        self.box_padding = 10

        self.image = wrap_in_box(render_text("", self.rect.w - self.box_padding, bg_color=gui_box_color),
                                 self.rect.size, 
                                 centery=True,
                                 padding=self.box_padding)

        self.cursor_size = (1, 17)
        self.cursor = pygame.Surface(self.cursor_size)

        self.cursor.fill(box_text_color)

    def addch(self, ch):
        self.input.insert(self.curs_pos, ch)
        self.curs_pos += 1

    def update(self):
        list = self.input[:]
        self.image = wrap_in_box(render_text("".join(list), self.rect.w - self.box_padding, bg_color=gui_box_color),
                                 self.rect.size,
                                 centery=True,
                                 padding=self.box_padding)

        # Display a cursor
        pos = utils.get_width("".join(list[:self.curs_pos]), 
                                 gui_font,
                                 gui_font_size,
                                 gui_italic,
                                 gui_bold)
        
        cursor_rect = self.cursor.get_rect()
        cursor_rect.centery = self.rect.h / 2 
        cursor_rect.x = pos + self.box_padding 

        self.image.blit(self.cursor, cursor_rect)

    def confirm(self):
        """Enter command"""
    
        self.parser.parse("".join(self.input))
        if len(self.input) > 0: # Add to history
            self.history.append(self.input[:])
            self.history_index += 1

        self.input = [] # Clear prompt for more messages
        self.curs_pos = 0
        self.history_index = len(self.history)

    def move_history(self, step):
        """Move in input history. Negative for back in history."""
        if len(self.history) is 0: # we have no history to move in
            return

        if self.history_index is len(self.history) - 1 and step > 0: # Scrolling down long enough should give a blank line
            self.input = []
            self.curs_pos = 0
            self.history_index = len(self.history)

        else:
            newhist = self.history_index + step
            if 0 <= newhist and newhist < len(self.history):
                self.history_index = newhist
                self.input = self.history[self.history_index][:] # Be sure that it is a copy we put in input
                self.curs_pos = len(self.input) # Position cursor at the end

        self.update()

    def _delete(self):
        try:
            del self.input[self.curs_pos]
        except IndexError:
            pass

    def delete_back(self):
        """Equivalent to backspace"""
        self.move(-1)
        self._delete()

    def delete_front(self):
        """Equivalent to delete"""
        self._delete()

    def move(self, step):
        """Move cursor by step. Negative step equals to the left."""
        newpos = self.curs_pos + step
        if 0 <= newpos and newpos <= len(self.input):
            self.curs_pos = newpos

def typable(keycode):
    """Is the user allowed to input keycode?"""
    if K_a <= keycode and keycode <= K_z:
        return True
    elif keycode == K_SPACE:
        return True
    elif K_0 <= keycode and keycode <= K_9:
        return True
    elif keycode == K_MINUS:
        return True
    else:
        return False

def render_text(s, 
                maxwidth,
                text_color=box_text_color, 
                bg_color=gui_box_color,
                font_size=gui_font_size, 
                font=gui_font, bold=gui_bold, 
                italic=gui_italic,
                box_size=None):

    text = utils.render_text(s, maxwidth, box_text_color, bg_color, gui_font_size, gui_font, gui_bold, gui_italic)
    return text

def wrap_in_box(surface, dimensions, centerx=False, centery=False, padding=20, box_color=gui_box_color):
    
    box = pygame.Surface(dimensions)

    colorkey = ((237, 34, 235))
    box.fill(colorkey)

    boxrect = box.get_rect()
    rect = surface.get_rect()
    
    # Fill the box everywhere except the corners
    vertical_rect = ((boxrect.x + padding, boxrect.y), (boxrect.w - 2*padding, boxrect.h))
    horizontal_rect = ((boxrect.x, boxrect.y + padding), (boxrect.w, boxrect.h - 2*padding)) 

    pygame.draw.rect(box, box_color, vertical_rect)
    pygame.draw.rect(box, box_color, horizontal_rect)

    # Draw rounded corners
    pygame.draw.circle(box, box_color, (padding, padding), padding)
    pygame.draw.circle(box, box_color, (boxrect.w - padding, padding), padding)
    pygame.draw.circle(box, box_color, (padding, boxrect.h - padding), padding)
    pygame.draw.circle(box, box_color, (boxrect.w - padding, boxrect.h - padding), padding)

   ## Draw a border
   #     pygame.draw.line(box, box_border_color, (padding, 0), (boxrect.w - padding, 0), box_border_w)
   #     pygame.draw.line(box, box_border_color, (padding, boxrect.h), (boxrect.w - padding, boxrect.h), box_border_w)
   #     pygame.draw.line(box, box_border_color, (0, padding), (0, boxrect.h - padding), box_border_w)
   #     pygame.draw.line(box, box_border_color, (boxrect.w, padding), (boxrect.w, boxrect.h - padding), box_border_w)
    
    
    #     tl = pygame.Rect(boxrect.topleft, (padding, padding))
    #     tr = pygame.Rect(boxrect.topleft, (padding, padding))
    #     tr.topright = boxrect.topright
    #     bl = pygame.Rect(boxrect.topleft, (padding, padding))
    #     bl.bottomleft = boxrect.bottomleft
    #     br = pygame.Rect(boxrect.topleft, (padding, padding))
    #     br.bottomright = boxrect.bottomright
    
    #     pygame.draw.arc(box, box_border_color, tl, math.pi/2, math.pi, box_border_w)
    #     pygame.draw.arc(box, box_border_color, tr, 0, math.pi/2, box_border_w)
    #     pygame.draw.arc(box, box_border_color, bl, math.pi, 3/2 * math.pi, box_border_w)
    #     pygame.draw.arc(box, box_border_color, br, -math.pi/2, 0, box_border_w)
    

    # Position the text
    if centerx:
        rect.centerx = boxrect.centerx
    else:
        rect.x = boxrect.x + padding
    if centery:
        rect.centery = boxrect.centery
    else:
        rect.y = boxrect.y + padding

    # Render the text
    box.blit(surface, rect)

    # Set colorkey and alpha
    box.convert()
    box.set_colorkey(colorkey, pygame.RLEACCEL)
    box.set_alpha(box_alpha)

    return box

    
